import {
  ConflictException,
  ForbiddenException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Not, Repository } from 'typeorm';
import { InstitucionPrograma } from './entity/institucion-programa.entity';
import { Operador } from '../operador/entity/operador.entity';
import { Programa } from './entity/programa.entity';
import { InstitucionService } from '../institucion/institucion.service';
import { InstitucionUsuarioService } from '../institucion-usuario/institucion-usuario.service';
import { Usuario } from '../usuario/entity/usuario.entity';

@Injectable()
export class InstitucionProgramaService {
  constructor(
    @InjectRepository(InstitucionPrograma)
    private institucionProgramaRepository: Repository<InstitucionPrograma>,
    @InjectRepository(Programa)
    private programaRepository: Repository<Programa>,
    private institucionService: InstitucionService,
    private institucionUsuarioService: InstitucionUsuarioService,
  ) {}

  create(programa: string): Promise<{ message: string }> {
    // Buscamos un registro con ese nombre
    return this.programaRepository
      .findOne({ where: { programa } })
      .then((existePrograma) => {
        // Sacamos error si existe
        if (existePrograma)
          throw new ConflictException('Ya existe este programa.');
        // Creamos y guardamos un registro
        return this.programaRepository.save(
          this.programaRepository.create({ programa }),
        );
      })
      .then(async (programa) => {
        const instituciones = await this.institucionService.findAll();

        // Asociamos la nueva infracción con las instituciones existentes
        for (let i = 0; i < instituciones.length; i++)
          await this.institucionProgramaRepository.save(
            this.institucionProgramaRepository.create({
              institucion: instituciones[i],
              programa,
            }),
          );
        return { message: 'Se creó correctamente un nuevo programa.' };
      });
  }

  findAll(): Promise<Programa[]> {
    return this.programaRepository.find({
      where: { id_programa: Not(1) },
      order: { programa: 'ASC' },
    });
  }

  async findAllMostrar(
    usuario: Usuario,
    id_institucion: number,
  ): Promise<InstitucionPrograma[]> {
    const institucion = await this.institucionService.findById(id_institucion);
    const institucionUsuario =
      await this.institucionUsuarioService.findByIdUsuarioIdInstitucion(
        usuario,
        institucion,
      );

    if (!institucionUsuario)
      throw new ForbiddenException(
        'No puedes acceder a esta información porque no peteneces a esta institución.',
      );
    return this.institucionProgramaRepository.find({
      where: { mostrar: true, institucion, programa: { id_programa: Not(1) } },
      order: { programa: { programa: 'ASC' } },
    });
  }

  async findAllOperador(operador: Operador): Promise<InstitucionPrograma[]> {
    return this.institucionProgramaRepository.find({
      where: {
        institucion: operador.institucion,
        programa: { id_programa: Not(1) },
      },
      order: { programa: { programa: 'ASC' } },
    });
  }

  findById(id_institucion_programa: number): Promise<InstitucionPrograma> {
    return this.institucionProgramaRepository
      .findOne({ where: { id_institucion_programa } })
      .then((institucionPrograma) => {
        if (!institucionPrograma)
          throw new NotFoundException('No existe esta institucion programa.');
        return institucionPrograma;
      });
  }

  findProgramaById(id_programa: number): Promise<Programa> {
    return this.programaRepository
      .findOne({ where: { id_programa } })
      .then((programa) => {
        if (!programa) throw new NotFoundException('No existe este programa.');
        return programa;
      });
  }

  findProgramaByPrograma(programa: string): Promise<Programa> {
    return this.programaRepository.findOne({ where: { programa } });
  }

  update(
    admin: Operador,
    attrs: Partial<InstitucionPrograma>,
  ): Promise<{ message: string }> {
    return this.findById(attrs.id_institucion_programa)
      .then((institucionPrograma) => {
        // Validamos que la institucion programa pertenezca a la institución del admin
        if (
          admin.institucion.id_institucion !=
          institucionPrograma.institucion.id_institucion
        )
          throw new ConflictException(
            'No puedes modificar la información este software porque no pertenece a tu institución.',
          );
        // Asignamos valores enviados al objeto
        Object.assign(institucionPrograma, attrs);
        // Guardamos cambios
        return this.institucionProgramaRepository.save(institucionPrograma);
      })
      .then((_) => ({ message: 'Se guardaron los cambios correctamente.' }));
  }
}
