import {
  Body,
  Controller,
  ForbiddenException,
  Get,
  Put,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionService } from './institucion.service';
import { ValidarUsuarioService } from '../validar-usuario/validar-usuario.service';
import { Operador } from '../operador/entity/operador.entity';
import { IdInstitucionDto } from '../dto/input/id-institucion.dto';
import { UpdateInstitucionDto } from './dto/input/update.dto';
import { MessageOutputDto } from '../dto/output/message.dto';
import { InstitucionOutputDto } from './dto/output/institucion.dto';
import { InstitucionMinOutputDto } from './dto/output/institucion-min.dto';
import { InstitucionesOutputDto } from './dto/output/instituciones.dto';

@Controller('institucion')
@ApiTags('institucion')
export class InstitucionController {
  constructor(
    private institucionService: InstitucionService,
    private validarUsuarioService: ValidarUsuarioService,
  ) {}

  @Serealize(InstitucionMinOutputDto)
  @Get('instituciones-activas')
  @ApiOperation({ description: 'Instituciones activas.' })
  activas() {
    return this.institucionService.findMinInfoAll(true);
  }

  @Serealize(InstitucionMinOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Todas las instituciones min.' })
  @ApiBearerAuth('jwt')
  get(@Request() req) {
    const superAdmin: Operador = req.user.operador;

    this.validarUsuarioService.validarSuperAdmin(superAdmin);
    return this.institucionService.findMinInfoAll();
  }

  @Serealize(InstitucionOutputDto)
  @Get('institucion')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Información de una institución.' })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id de la institución.',
    name: 'id_institucion',
    type: 'string',
  })
  institucion(@Request() req, @Query() query: IdInstitucionDto) {
    const admin: Operador = req.user.operador;

    this.validarUsuarioService.validarSuperAdminAdmin(admin);
    return this.institucionService.findFullInfoById(
      admin,
      parseInt(query.id_institucion),
    );
  }

  @Serealize(InstitucionesOutputDto)
  @Get('instituciones')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Todas las instituciones con responsable.' })
  @ApiBearerAuth('jwt')
  instituciones(@Request() req) {
    const superAdmin: Operador = req.user.operador;

    this.validarUsuarioService.validarSuperAdmin(superAdmin);
    return this.institucionService.findMinInfoAll(false, true);
  }

  @Serealize(MessageOutputDto)
  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Actualizar la información de una institución.' })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description:
      'Variables que necesita el endpoint. Las variables con "_" al inicio son opcionales.',
    examples: {
      ejemplo: {
        value: {
          id_institucion: 200,
          _activo: true,
          _correo: '',
          _dias_multa_retraso: 7,
          _responsable: '',
          _telefono: '',
          _tiempo_entrega: 15,
          _tiempo_prestamo: 120,
          _tiempo_recoger: 10,
          _ubicacion: '',
        },
      },
    },
  })
  update(@Request() req, @Body() body: UpdateInstitucionDto) {
    const admin: Operador = req.user.operador;

    this.validarUsuarioService.validarSuperAdminAdmin(admin);
    return this.institucionService.update(admin, body);
  }
}
