import {
  ConflictException,
  forwardRef,
  Inject,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FindOptionsWhere, Repository } from 'typeorm';
import { Institucion } from '../institucion/entity/institucion.entity';
import { Modulo } from './entity/modulo.entity';
import { InformacionModuloView } from './entity/views/informacion-modulo.view';
import { MinInformacionModuloView } from './entity/views/min-informacion-modulo.view';
import { Operador } from '../operador/entity/operador.entity';
import { InstitucionService } from '../institucion/institucion.service';
import { ModuloMotivoService } from '../modulo-motivo/modulo-motivo.service';

@Injectable()
export class ModuloService {
  constructor(
    @InjectRepository(Modulo) private repository: Repository<Modulo>,
    @InjectRepository(InformacionModuloView)
    private informacionModuloView: Repository<InformacionModuloView>,
    @InjectRepository(MinInformacionModuloView)
    private minInformacionModuloView: Repository<MinInformacionModuloView>,
    @Inject(forwardRef(() => ModuloMotivoService))
    private moduloMotivoService: ModuloMotivoService,
    private institucionService: InstitucionService,
  ) {}

  create(admin: Operador, modulo: string) {
    // Buscamos una modulo con ese nombre en la institución del admin
    // que realiza esta acción
    return this.findModulo(admin.institucion, modulo, true)
      .then((_) =>
        // Creamos y guardamos un registro
        this.repository.save(
          this.repository.create({
            institucion: admin.institucion,
            modulo,
          }),
        ),
      )
      .then((_) => ({ message: 'Se creó correctamente un nuevo módulo.' }));
  }

  findFullInfoById(id_modulo: number) {
    return this.repository.findOne({ where: { id_modulo } }).then((modulo) => {
      if (!modulo) throw new NotFoundException('No existe este id módulo.');
      return modulo;
    });
  }

  findById(id_modulo: number) {
    return this.informacionModuloView
      .findOne({ where: { id_modulo } })
      .then((infoModulo) => {
        if (!infoModulo)
          throw new NotFoundException('No existe este id módulo.');
        return this.repository.create({
          id_modulo: infoModulo.id_modulo,
          institucion: { id_institucion: infoModulo.id_institucion },
        });
      });
  }

  findMinInfoModuloAll(id_institucion: number, activo = 0) {
    return this.institucionService
      .findById(id_institucion)
      .then((institucion) => {
        const busqueda: FindOptionsWhere<MinInformacionModuloView> = {
          id_institucion: institucion.id_institucion,
        };

        if (activo) busqueda.activo_modulo = activo;
        return this.minInformacionModuloView.find({ where: busqueda });
      })
      .then((infoModulos) => {
        const modulos: Modulo[] = [];

        for (let i = 0; i < infoModulos.length; i++)
          modulos.push(
            this.repository.create({
              id_modulo: infoModulos[i].id_modulo,
              activo: infoModulos[i].activo_modulo === 1,
              modulo: infoModulos[i].modulo,
              institucion: {
                id_institucion: infoModulos[i].id_institucion,
                institucion: infoModulos[i].institucion,
              },
            }),
          );
        return modulos;
      });
  }

  findMinInfoModuloById(id_modulo: number) {
    return this.minInformacionModuloView
      .findOne({ where: { id_modulo } })
      .then((infoModulo) => {
        if (!infoModulo)
          throw new NotFoundException('No existe este id módulo.');
        return this.repository.create({
          id_modulo: infoModulo.id_modulo,
          activo: infoModulo.activo_modulo === 1,
          modulo: infoModulo.modulo,
          institucion: {
            id_institucion: infoModulo.id_institucion,
            institucion: infoModulo.institucion,
          },
        });
      });
  }

  findModulo(institucion: Institucion, modulo: string, validarExiste = false) {
    return this.minInformacionModuloView
      .findOne({
        where: { id_institucion: institucion.id_institucion, modulo },
      })
      .then((infoModulo) => {
        if (!infoModulo) return null;
        if (validarExiste)
          throw new ConflictException(
            'Ya existe un módulo en esta institución con este nombre, intente con otro.',
          );
        return this.repository.create({
          id_modulo: infoModulo.id_modulo,
          institucion: { id_institucion: infoModulo.id_institucion },
        });
      });
  }

  update(
    admin: Operador,
    attrs: Partial<Modulo>,
    motivo?: string,
    numero_alumnos?: number,
    fecha_creacion?: string,
  ) {
    if (
      (!motivo || !numero_alumnos || !fecha_creacion) &&
      (motivo || numero_alumnos || fecha_creacion)
    )
      throw new ConflictException('No se mandó toda la información necesaria.');
    return this.findById(attrs.id_modulo)
      .then(async (modulo) => {
        // Validamos que el modulo enviado sea del admin que realiza esta acción
        if (
          admin.institucion.id_institucion != modulo.institucion.id_institucion
        )
          throw new ConflictException(
            'No puedes actualizar la información de este módulo porque no le pertenece a tu institución.',
          );
        // Buscamos un modulo en esa institución con ese nombre
        if (attrs.modulo)
          await this.findModulo(modulo.institucion, attrs.modulo, true);
        // Asignamos valores enviados al objeto
        Object.assign(modulo, attrs);
        if (motivo)
          await this.moduloMotivoService.create(
            admin,
            modulo,
            motivo,
            numero_alumnos,
            fecha_creacion,
          );
        // Guardamos cambios
        return this.repository.save(modulo);
      })
      .then((_) => ({ message: 'Se guardaron los cambios correctamente.' }));
  }
}
