import {
  Body,
  Controller,
  Get,
  Put,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { PasscodeService } from './passcode.service';
import { ValidarUsuarioService } from '../validar-usuario/validar-usuario.service';
import { Modulo } from '../modulo/entity/modulo.entity';
import { Operador } from '../operador/entity/operador.entity';
import { DevolucionDto } from './dto/input/devolucion.dto';
import { NotificacionDto } from './dto/input/notificacion.dto';
import { PrestamoDto } from './dto/input/prestamo.dto';
import { CodeOutputDto } from './dto/output/code.dto';
import { PasscodeOutputDto } from './dto/output/passcode.dto';

@Controller('passcode')
@ApiTags('passcode')
export class PasscodeController {
  constructor(
    private passcodeService: PasscodeService,
    private validarUsuarioService: ValidarUsuarioService,
  ) {}

  @Serealize(PasscodeOutputDto)
  @Get('prestamo')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Endpoint que verifica la existencia y validez del passcode.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Passcode del préstamo',
    name: 'passcode',
    type: 'string',
  })
  @ApiQuery({
    description: 'id_locker, un locker hace referencia a un carrito.',
    name: 'id_locker',
    type: 'string',
  })
  prestamo(@Request() req, @Query() query: PrestamoDto) {
    const operador: Operador = req.user.operador;

    this.validarUsuarioService.validarSoloOperador(operador);
    return this.passcodeService.findByPasscode(
      query.passcode,
      false,
      query.id_locker,
    );
  }

  @Serealize(CodeOutputDto)
  @Put('notificacion')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Actualizar el status del equipo a "En uso" por medio del autoprestamo.',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Ambas variables son obligatorias.',
    examples: { ejemplo: { value: { passcode: 123456789012, estatus: 1 } } },
  })
  autoprestamo(@Request() req, @Body() body: NotificacionDto) {
    const operador: Operador = req.user.operador;
    const modulo: Modulo = req.user.modulo;

    this.validarUsuarioService.validarSoloOperador(operador);
    return this.passcodeService.entregarEquipo(
      operador,
      modulo,
      body.passcode,
      body.estatus,
    );
  }

  @Serealize(CodeOutputDto)
  @Put('devolucion')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Endpoint que da por terminado un préstamo.' })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Solo necesitamos el passcode.',
    examples: {
      ejemplo: {
        value: {
          passcode: 123456789012,
        },
      },
    },
  })
  devolucion(@Request() req, @Body() body: DevolucionDto) {
    const operador: Operador = req.user.operador;
    const modulo: Modulo = req.user.modulo;

    this.validarUsuarioService.validarSoloOperador(operador);
    return this.passcodeService.devolverEquipo(operador, modulo, body.passcode);
  }
}
